"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

from typing import Any, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="ValidationError")


@_attrs_define
class ValidationError:
    """
    Attributes:
        loc (List[Union[int, str]]):
        msg (str):
        type (str):
    """

    loc: List[Union[int, str]]
    msg: str
    type: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        loc = []
        for loc_item_data in self.loc:
            loc_item: Union[int, str]
            loc_item = loc_item_data
            loc.append(loc_item)

        msg = self.msg

        type = self.type

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "loc": loc,
                "msg": msg,
                "type": type,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        loc = []
        _loc = d.pop("loc")
        for loc_item_data in _loc:

            def _parse_loc_item(data: object) -> Union[int, str]:
                return cast(Union[int, str], data)

            loc_item = _parse_loc_item(loc_item_data)

            loc.append(loc_item)

        msg = d.pop("msg")

        type = d.pop("type")

        validation_error = cls(
            loc=loc,
            msg=msg,
            type=type,
        )

        validation_error.additional_properties = d
        return validation_error

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""
This module contains metrics for usage with the Hugging Face Trainer.

IMPORTANT:
- This is a shared file between OrcaLib and the Orca SDK.
- Please ensure that it does not have any dependencies on the OrcaLib code.
- Make sure to edit this file in orcalib/shared and NOT in orca_sdk, since it will be overwritten there.

"""

from typing import Literal, Tuple, TypedDict

import numpy as np
from numpy.typing import NDArray
from scipy.special import softmax
from sklearn.metrics import accuracy_score, auc, f1_score, log_loss
from sklearn.metrics import precision_recall_curve as sklearn_precision_recall_curve
from sklearn.metrics import roc_auc_score
from sklearn.metrics import roc_curve as sklearn_roc_curve
from transformers.trainer_utils import EvalPrediction


class ClassificationMetrics(TypedDict):
    accuracy: float
    f1_score: float
    roc_auc: float | None  # receiver operating characteristic area under the curve (if all classes are present)
    pr_auc: float | None  # precision-recall area under the curve (only for binary classification)
    log_loss: float  # cross-entropy loss for probabilities


def compute_classifier_metrics(eval_pred: EvalPrediction) -> ClassificationMetrics:
    """
    Compute standard metrics for classifier with Hugging Face Trainer.

    Args:
        eval_pred: The predictions containing logits and expected labels as given by the Trainer.

    Returns:
        A dictionary containing the accuracy, f1 score, and ROC AUC score.
    """
    logits, references = eval_pred
    if isinstance(logits, tuple):
        logits = logits[0]
    if not isinstance(logits, np.ndarray):
        raise ValueError("Logits must be a numpy array")
    if not isinstance(references, np.ndarray):
        raise ValueError(
            "Multiple label columns found, use the `label_names` training argument to specify which one to use"
        )

    if not (logits > 0).all():
        # convert logits to probabilities with softmax if necessary
        probabilities = softmax(logits)
    elif not np.allclose(logits.sum(-1, keepdims=True), 1.0):
        # convert logits to probabilities through normalization if necessary
        probabilities = logits / logits.sum(-1, keepdims=True)
    else:
        probabilities = logits

    return classification_scores(references, probabilities)


def classification_scores(
    references: NDArray[np.int64],
    probabilities: NDArray[np.float32],
    average: Literal["micro", "macro", "weighted", "binary"] | None = None,
    multi_class: Literal["ovr", "ovo"] = "ovr",
) -> ClassificationMetrics:
    if probabilities.ndim == 1:
        # convert 1D probabilities (binary) to 2D logits
        probabilities = np.column_stack([1 - probabilities, probabilities])
    elif probabilities.ndim == 2:
        if probabilities.shape[1] < 2:
            raise ValueError("Use a different metric function for regression tasks")
    else:
        raise ValueError("Probabilities must be 1 or 2 dimensional")

    predictions = np.argmax(probabilities, axis=-1)

    num_classes_references = len(set(references))
    num_classes_predictions = len(set(predictions))

    if average is None:
        average = "binary" if num_classes_references == 2 else "weighted"

    accuracy = accuracy_score(references, predictions)
    f1 = f1_score(references, predictions, average=average)
    loss = log_loss(references, probabilities)

    if num_classes_references == num_classes_predictions:
        # special case for binary classification: https://github.com/scikit-learn/scikit-learn/issues/20186
        if num_classes_references == 2:
            roc_auc = roc_auc_score(references, probabilities[:, 1])
            precisions, recalls, _ = calculate_pr_curve(references, probabilities[:, 1])
            pr_auc = auc(recalls, precisions)
        else:
            roc_auc = roc_auc_score(references, probabilities, multi_class=multi_class)
            pr_auc = None
    else:
        roc_auc = None
        pr_auc = None

    return {
        "accuracy": float(accuracy),
        "f1_score": float(f1),
        "roc_auc": float(roc_auc) if roc_auc is not None else None,
        "pr_auc": float(pr_auc) if pr_auc is not None else None,
        "log_loss": float(loss),
    }


def calculate_pr_curve(
    references: NDArray[np.int64],
    probabilities: NDArray[np.float32],
    max_length: int = 100,
) -> Tuple[NDArray[np.float32], NDArray[np.float32], NDArray[np.float32]]:
    if probabilities.ndim == 1:
        probabilities_slice = probabilities
    elif probabilities.ndim == 2:
        probabilities_slice = probabilities[:, 1]
    else:
        raise ValueError("Probabilities must be 1 or 2 dimensional")

    if len(probabilities_slice) != len(references):
        raise ValueError("Probabilities and references must have the same length")

    precisions, recalls, thresholds = sklearn_precision_recall_curve(references, probabilities_slice)

    # Convert all arrays to float32 immediately after getting them
    precisions = precisions.astype(np.float32)
    recalls = recalls.astype(np.float32)
    thresholds = thresholds.astype(np.float32)

    # Concatenate with 0 to include the lowest threshold
    thresholds = np.concatenate(([0], thresholds))

    # Sort by threshold
    sorted_indices = np.argsort(thresholds)
    thresholds = thresholds[sorted_indices]
    precisions = precisions[sorted_indices]
    recalls = recalls[sorted_indices]

    if len(precisions) > max_length:
        new_thresholds = np.linspace(0, 1, max_length, dtype=np.float32)
        new_precisions = np.interp(new_thresholds, thresholds, precisions)
        new_recalls = np.interp(new_thresholds, thresholds, recalls)
        thresholds = new_thresholds
        precisions = new_precisions
        recalls = new_recalls

    return precisions.astype(np.float32), recalls.astype(np.float32), thresholds.astype(np.float32)


def calculate_roc_curve(
    references: NDArray[np.int64],
    probabilities: NDArray[np.float32],
    max_length: int = 100,
) -> Tuple[NDArray[np.float32], NDArray[np.float32], NDArray[np.float32]]:
    if probabilities.ndim == 1:
        probabilities_slice = probabilities
    elif probabilities.ndim == 2:
        probabilities_slice = probabilities[:, 1]
    else:
        raise ValueError("Probabilities must be 1 or 2 dimensional")

    if len(probabilities_slice) != len(references):
        raise ValueError("Probabilities and references must have the same length")

    # Convert probabilities to float32 before calling sklearn_roc_curve
    probabilities_slice = probabilities_slice.astype(np.float32)
    fpr, tpr, thresholds = sklearn_roc_curve(references, probabilities_slice)

    # Convert all arrays to float32 immediately after getting them
    fpr = fpr.astype(np.float32)
    tpr = tpr.astype(np.float32)
    thresholds = thresholds.astype(np.float32)

    # We set the first threshold to 1.0 instead of inf for reasonable values in interpolation
    thresholds[0] = 1.0

    # Sort by threshold
    sorted_indices = np.argsort(thresholds)
    thresholds = thresholds[sorted_indices]
    fpr = fpr[sorted_indices]
    tpr = tpr[sorted_indices]

    if len(fpr) > max_length:
        new_thresholds = np.linspace(0, 1, max_length, dtype=np.float32)
        new_fpr = np.interp(new_thresholds, thresholds, fpr)
        new_tpr = np.interp(new_thresholds, thresholds, tpr)
        thresholds = new_thresholds
        fpr = new_fpr
        tpr = new_tpr

    return fpr.astype(np.float32), tpr.astype(np.float32), thresholds.astype(np.float32)



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

from typing import Any, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="CascadeEditSuggestionsRequest")


@_attrs_define
class CascadeEditSuggestionsRequest:
    """
    Attributes:
        old_label (int):
        new_label (int):
        max_neighbors (Union[Unset, int]):  Default: 50.
        max_validation_neighbors (Union[Unset, int]):  Default: 10.
        similarity_threshold (Union[None, Unset, float]):
        only_if_has_old_label (Union[Unset, bool]):  Default: True.
        exclude_if_new_label (Union[Unset, bool]):  Default: True.
        suggestion_cooldown_time (Union[Unset, float]):  Default: 86400.0.
        label_confirmation_cooldown_time (Union[Unset, float]):  Default: 604800.0.
    """

    old_label: int
    new_label: int
    max_neighbors: Union[Unset, int] = 50
    max_validation_neighbors: Union[Unset, int] = 10
    similarity_threshold: Union[None, Unset, float] = UNSET
    only_if_has_old_label: Union[Unset, bool] = True
    exclude_if_new_label: Union[Unset, bool] = True
    suggestion_cooldown_time: Union[Unset, float] = 86400.0
    label_confirmation_cooldown_time: Union[Unset, float] = 604800.0
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        old_label = self.old_label

        new_label = self.new_label

        max_neighbors = self.max_neighbors

        max_validation_neighbors = self.max_validation_neighbors

        similarity_threshold: Union[None, Unset, float]
        if isinstance(self.similarity_threshold, Unset):
            similarity_threshold = UNSET
        else:
            similarity_threshold = self.similarity_threshold

        only_if_has_old_label = self.only_if_has_old_label

        exclude_if_new_label = self.exclude_if_new_label

        suggestion_cooldown_time = self.suggestion_cooldown_time

        label_confirmation_cooldown_time = self.label_confirmation_cooldown_time

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "old_label": old_label,
                "new_label": new_label,
            }
        )
        if max_neighbors is not UNSET:
            field_dict["max_neighbors"] = max_neighbors
        if max_validation_neighbors is not UNSET:
            field_dict["max_validation_neighbors"] = max_validation_neighbors
        if similarity_threshold is not UNSET:
            field_dict["similarity_threshold"] = similarity_threshold
        if only_if_has_old_label is not UNSET:
            field_dict["only_if_has_old_label"] = only_if_has_old_label
        if exclude_if_new_label is not UNSET:
            field_dict["exclude_if_new_label"] = exclude_if_new_label
        if suggestion_cooldown_time is not UNSET:
            field_dict["suggestion_cooldown_time"] = suggestion_cooldown_time
        if label_confirmation_cooldown_time is not UNSET:
            field_dict["label_confirmation_cooldown_time"] = label_confirmation_cooldown_time

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        old_label = d.pop("old_label")

        new_label = d.pop("new_label")

        max_neighbors = d.pop("max_neighbors", UNSET)

        max_validation_neighbors = d.pop("max_validation_neighbors", UNSET)

        def _parse_similarity_threshold(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        similarity_threshold = _parse_similarity_threshold(d.pop("similarity_threshold", UNSET))

        only_if_has_old_label = d.pop("only_if_has_old_label", UNSET)

        exclude_if_new_label = d.pop("exclude_if_new_label", UNSET)

        suggestion_cooldown_time = d.pop("suggestion_cooldown_time", UNSET)

        label_confirmation_cooldown_time = d.pop("label_confirmation_cooldown_time", UNSET)

        cascade_edit_suggestions_request = cls(
            old_label=old_label,
            new_label=new_label,
            max_neighbors=max_neighbors,
            max_validation_neighbors=max_validation_neighbors,
            similarity_threshold=similarity_threshold,
            only_if_has_old_label=only_if_has_old_label,
            exclude_if_new_label=exclude_if_new_label,
            suggestion_cooldown_time=suggestion_cooldown_time,
            label_confirmation_cooldown_time=label_confirmation_cooldown_time,
        )

        cascade_edit_suggestions_request.additional_properties = d
        return cascade_edit_suggestions_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class PretrainedEmbeddingModelName(str, Enum):
    BGE_BASE = "BGE_BASE"
    CDE_SMALL = "CDE_SMALL"
    CLIP_BASE = "CLIP_BASE"
    DISTILBERT = "DISTILBERT"
    E5_LARGE = "E5_LARGE"
    GIST_LARGE = "GIST_LARGE"
    GTE_BASE = "GTE_BASE"
    GTE_SMALL = "GTE_SMALL"
    MXBAI_LARGE = "MXBAI_LARGE"
    QWEN2_1_5B = "QWEN2_1_5B"

    def __str__(self) -> str:
        return str(self.value)



"""
IMPORTANT:
- This is a shared file between OrcaLib and the Orca SDK.
- Please ensure that it does not have any dependencies on the OrcaLib code.
- Make sure to edit this file in orcalib/shared and NOT in orca_sdk, since it will be overwritten there.
"""

from typing import Literal

import numpy as np
import pytest

from .metrics import (
    EvalPrediction,
    calculate_pr_curve,
    calculate_roc_curve,
    classification_scores,
    compute_classifier_metrics,
    softmax,
)


def test_binary_metrics():
    y_true = np.array([0, 1, 1, 0, 1])
    y_score = np.array([0.1, 0.9, 0.8, 0.3, 0.2])

    metrics = classification_scores(y_true, y_score)

    assert metrics["accuracy"] == 0.8
    assert metrics["f1_score"] == 0.8
    assert metrics["roc_auc"] is not None
    assert metrics["roc_auc"] > 0.8
    assert metrics["roc_auc"] < 1.0
    assert metrics["pr_auc"] is not None
    assert metrics["pr_auc"] > 0.8
    assert metrics["pr_auc"] < 1.0
    assert metrics["log_loss"] is not None
    assert metrics["log_loss"] > 0.0


def test_multiclass_metrics_with_2_classes():
    y_true = np.array([0, 1, 1, 0, 1])
    y_score = np.array([[0.9, 0.1], [0.1, 0.9], [0.2, 0.8], [0.7, 0.3], [0.8, 0.2]])

    metrics = classification_scores(y_true, y_score)

    assert metrics["accuracy"] == 0.8
    assert metrics["f1_score"] == 0.8
    assert metrics["roc_auc"] is not None
    assert metrics["roc_auc"] > 0.8
    assert metrics["roc_auc"] < 1.0
    assert metrics["pr_auc"] is not None
    assert metrics["pr_auc"] > 0.8
    assert metrics["pr_auc"] < 1.0
    assert metrics["log_loss"] is not None
    assert metrics["log_loss"] > 0.0


@pytest.mark.parametrize(
    "average, multiclass",
    [("micro", "ovr"), ("macro", "ovr"), ("weighted", "ovr"), ("micro", "ovo"), ("macro", "ovo"), ("weighted", "ovo")],
)
def test_multiclass_metrics_with_3_classes(
    average: Literal["micro", "macro", "weighted"], multiclass: Literal["ovr", "ovo"]
):
    y_true = np.array([0, 1, 1, 0, 2])
    y_score = np.array([[0.9, 0.1, 0.0], [0.1, 0.9, 0.0], [0.2, 0.8, 0.0], [0.7, 0.3, 0.0], [0.0, 0.0, 1.0]])

    metrics = classification_scores(y_true, y_score, average=average, multi_class=multiclass)

    assert metrics["accuracy"] == 1.0
    assert metrics["f1_score"] == 1.0
    assert metrics["roc_auc"] is not None
    assert metrics["roc_auc"] > 0.8
    assert metrics["pr_auc"] is None
    assert metrics["log_loss"] is not None
    assert metrics["log_loss"] > 0.0


def test_does_not_modify_logits_unless_necessary():
    logits = np.array([[0.1, 0.9], [0.2, 0.8], [0.7, 0.3], [0.8, 0.2]])
    references = np.array([0, 1, 0, 1])
    metrics = compute_classifier_metrics(EvalPrediction(logits, references))
    assert metrics["log_loss"] == classification_scores(references, logits)["log_loss"]


def test_normalizes_logits_if_necessary():
    logits = np.array([[1.2, 3.9], [1.2, 5.8], [1.2, 2.7], [1.2, 1.3]])
    references = np.array([0, 1, 0, 1])
    metrics = compute_classifier_metrics(EvalPrediction(logits, references))
    assert (
        metrics["log_loss"] == classification_scores(references, logits / logits.sum(axis=1, keepdims=True))["log_loss"]
    )


def test_softmaxes_logits_if_necessary():
    logits = np.array([[-1.2, 3.9], [1.2, -5.8], [1.2, 2.7], [1.2, 1.3]])
    references = np.array([0, 1, 0, 1])
    metrics = compute_classifier_metrics(EvalPrediction(logits, references))
    assert metrics["log_loss"] == classification_scores(references, softmax(logits))["log_loss"]


def test_precision_recall_curve():
    y_true = np.array([0, 1, 1, 0, 1])
    y_score = np.array([0.1, 0.9, 0.8, 0.6, 0.2])

    precision, recall, thresholds = calculate_pr_curve(y_true, y_score)
    assert precision is not None
    assert recall is not None
    assert thresholds is not None

    assert len(precision) == len(recall) == len(thresholds) == 6
    assert precision[0] == 0.6
    assert recall[0] == 1.0
    assert precision[-1] == 1.0
    assert recall[-1] == 0.0

    # test that thresholds are sorted
    assert np.all(np.diff(thresholds) >= 0)


def test_roc_curve():
    y_true = np.array([0, 1, 1, 0, 1])
    y_score = np.array([0.1, 0.9, 0.8, 0.6, 0.2])

    fpr, tpr, thresholds = calculate_roc_curve(y_true, y_score)
    assert fpr is not None
    assert tpr is not None
    assert thresholds is not None

    assert len(fpr) == len(tpr) == len(thresholds) == 6
    assert fpr[0] == 1.0
    assert tpr[0] == 1.0
    assert fpr[-1] == 0.0
    assert tpr[-1] == 0.0

    # test that thresholds are sorted
    assert np.all(np.diff(thresholds) >= 0)


def test_precision_recall_curve_max_length():
    y_true = np.array([0, 1, 1, 0, 1])
    y_score = np.array([0.1, 0.9, 0.8, 0.6, 0.2])

    precision, recall, thresholds = calculate_pr_curve(y_true, y_score, max_length=5)
    assert len(precision) == len(recall) == len(thresholds) == 5

    assert precision[0] == 0.6
    assert recall[0] == 1.0
    assert precision[-1] == 1.0
    assert recall[-1] == 0.0

    # test that thresholds are sorted
    assert np.all(np.diff(thresholds) >= 0)


def test_roc_curve_max_length():
    y_true = np.array([0, 1, 1, 0, 1])
    y_score = np.array([0.1, 0.9, 0.8, 0.6, 0.2])

    fpr, tpr, thresholds = calculate_roc_curve(y_true, y_score, max_length=5)
    assert len(fpr) == len(tpr) == len(thresholds) == 5
    assert fpr[0] == 1.0
    assert tpr[0] == 1.0
    assert fpr[-1] == 0.0
    assert tpr[-1] == 0.0

    # test that thresholds are sorted
    assert np.all(np.diff(thresholds) >= 0)



import logging
import os
from uuid import uuid4

import numpy as np
import pytest
from datasets.arrow_dataset import Dataset

from .classification_model import ClassificationModel
from .datasource import Datasource
from .embedding_model import PretrainedEmbeddingModel
from .memoryset import LabeledMemoryset

logging.basicConfig(level=logging.INFO, format="%(asctime)s [%(levelname)s] %(message)s")


SKIP_IN_GITHUB_ACTIONS = os.getenv("GITHUB_ACTIONS") == "true"


def test_create_model(model: ClassificationModel, readonly_memoryset: LabeledMemoryset):
    assert model is not None
    assert model.name == "test_model"
    assert model.memoryset == readonly_memoryset
    assert model.num_classes == 2
    assert model.memory_lookup_count == 3


def test_create_model_already_exists_error(readonly_memoryset, model: ClassificationModel):
    with pytest.raises(ValueError):
        ClassificationModel.create("test_model", readonly_memoryset)
    with pytest.raises(ValueError):
        ClassificationModel.create("test_model", readonly_memoryset, if_exists="error")


def test_create_model_already_exists_return(readonly_memoryset, model: ClassificationModel):
    with pytest.raises(ValueError):
        ClassificationModel.create("test_model", readonly_memoryset, if_exists="open", head_type="MMOE")

    with pytest.raises(ValueError):
        ClassificationModel.create("test_model", readonly_memoryset, if_exists="open", memory_lookup_count=37)

    with pytest.raises(ValueError):
        ClassificationModel.create("test_model", readonly_memoryset, if_exists="open", num_classes=19)

    with pytest.raises(ValueError):
        ClassificationModel.create("test_model", readonly_memoryset, if_exists="open", min_memory_weight=0.77)

    new_model = ClassificationModel.create("test_model", readonly_memoryset, if_exists="open")
    assert new_model is not None
    assert new_model.name == "test_model"
    assert new_model.memoryset == readonly_memoryset
    assert new_model.num_classes == 2
    assert new_model.memory_lookup_count == 3


def test_create_model_unauthenticated(unauthenticated, readonly_memoryset: LabeledMemoryset):
    with pytest.raises(ValueError, match="Invalid API key"):
        ClassificationModel.create("test_model", readonly_memoryset)


def test_get_model(model: ClassificationModel):
    fetched_model = ClassificationModel.open(model.name)
    assert fetched_model is not None
    assert fetched_model.id == model.id
    assert fetched_model.name == model.name
    assert fetched_model.num_classes == 2
    assert fetched_model.memory_lookup_count == 3
    assert fetched_model == model


def test_get_model_unauthenticated(unauthenticated):
    with pytest.raises(ValueError, match="Invalid API key"):
        ClassificationModel.open("test_model")


def test_get_model_invalid_input():
    with pytest.raises(ValueError, match="Invalid input"):
        ClassificationModel.open("not valid id")


def test_get_model_not_found():
    with pytest.raises(LookupError):
        ClassificationModel.open(str(uuid4()))


def test_get_model_unauthorized(unauthorized, model: ClassificationModel):
    with pytest.raises(LookupError):
        ClassificationModel.open(model.name)


def test_list_models(model: ClassificationModel):
    models = ClassificationModel.all()
    assert len(models) > 0
    assert any(model.name == model.name for model in models)


def test_list_models_unauthenticated(unauthenticated):
    with pytest.raises(ValueError, match="Invalid API key"):
        ClassificationModel.all()


def test_list_models_unauthorized(unauthorized, model: ClassificationModel):
    assert ClassificationModel.all() == []


def test_update_model(model: ClassificationModel):
    model.update_metadata(description="New description")
    assert model.description == "New description"


def test_update_model_no_description(model: ClassificationModel):
    assert model.description is not None
    model.update_metadata(description=None)
    assert model.description is None


def test_delete_model(readonly_memoryset: LabeledMemoryset):
    ClassificationModel.create("model_to_delete", LabeledMemoryset.open(readonly_memoryset.name))
    assert ClassificationModel.open("model_to_delete")
    ClassificationModel.drop("model_to_delete")
    with pytest.raises(LookupError):
        ClassificationModel.open("model_to_delete")


def test_delete_model_unauthenticated(unauthenticated, model: ClassificationModel):
    with pytest.raises(ValueError, match="Invalid API key"):
        ClassificationModel.drop(model.name)


def test_delete_model_not_found():
    with pytest.raises(LookupError):
        ClassificationModel.drop(str(uuid4()))
    # ignores error if specified
    ClassificationModel.drop(str(uuid4()), if_not_exists="ignore")


def test_delete_model_unauthorized(unauthorized, model: ClassificationModel):
    with pytest.raises(LookupError):
        ClassificationModel.drop(model.name)


def test_delete_memoryset_before_model_constraint_violation(hf_dataset):
    memoryset = LabeledMemoryset.from_hf_dataset("test_memoryset_delete_before_model", hf_dataset)
    ClassificationModel.create("test_model_delete_before_memoryset", memoryset)
    with pytest.raises(RuntimeError):
        LabeledMemoryset.drop(memoryset.id)


def test_evaluate(model, eval_datasource: Datasource):
    result = model.evaluate(eval_datasource)
    assert result is not None
    assert isinstance(result, dict)
    # And anomaly score statistics are present and valid
    assert isinstance(result["anomaly_score_mean"], float)
    assert isinstance(result["anomaly_score_median"], float)
    assert isinstance(result["anomaly_score_variance"], float)
    assert -1.0 <= result["anomaly_score_mean"] <= 1.0
    assert -1.0 <= result["anomaly_score_median"] <= 1.0
    assert -1.0 <= result["anomaly_score_variance"] <= 1.0
    assert isinstance(result["accuracy"], float)
    assert isinstance(result["f1_score"], float)
    assert isinstance(result["loss"], float)
    assert len(result["precision_recall_curve"]["thresholds"]) == 4
    assert len(result["precision_recall_curve"]["precisions"]) == 4
    assert len(result["precision_recall_curve"]["recalls"]) == 4
    assert len(result["roc_curve"]["thresholds"]) == 4
    assert len(result["roc_curve"]["false_positive_rates"]) == 4
    assert len(result["roc_curve"]["true_positive_rates"]) == 4


def test_evaluate_combined(model, eval_datasource: Datasource, eval_dataset: Dataset):
    result_datasource = model.evaluate(eval_datasource)

    result_dataset = model.evaluate(eval_dataset)

    for result in [result_datasource, result_dataset]:
        assert result is not None
        assert isinstance(result, dict)
        assert isinstance(result["accuracy"], float)
        assert isinstance(result["f1_score"], float)
        assert isinstance(result["loss"], float)
        assert np.allclose(result["accuracy"], 0.5)
        assert np.allclose(result["f1_score"], 0.5)

        assert isinstance(result["precision_recall_curve"]["thresholds"], list)
        assert isinstance(result["precision_recall_curve"]["precisions"], list)
        assert isinstance(result["precision_recall_curve"]["recalls"], list)
        assert isinstance(result["roc_curve"]["thresholds"], list)
        assert isinstance(result["roc_curve"]["false_positive_rates"], list)
        assert isinstance(result["roc_curve"]["true_positive_rates"], list)

        assert np.allclose(result["roc_curve"]["thresholds"], [0.0, 0.8155114054679871, 0.834095299243927, 1.0])
        assert np.allclose(result["roc_curve"]["false_positive_rates"], [1.0, 0.5, 0.0, 0.0])
        assert np.allclose(result["roc_curve"]["true_positive_rates"], [1.0, 0.5, 0.5, 0.0])
        assert np.allclose(result["roc_curve"]["auc"], 0.625)

        assert np.allclose(
            result["precision_recall_curve"]["thresholds"], [0.0, 0.0, 0.8155114054679871, 0.834095299243927]
        )
        assert np.allclose(result["precision_recall_curve"]["precisions"], [0.5, 0.5, 1.0, 1.0])
        assert np.allclose(result["precision_recall_curve"]["recalls"], [1.0, 0.5, 0.5, 0.0])
        assert np.allclose(result["precision_recall_curve"]["auc"], 0.75)


def test_evaluate_with_telemetry(model):
    samples = [
        {"text": "chicken noodle soup is the best", "label": 1},
        {"text": "cats are cute", "label": 0},
    ]
    eval_datasource = Datasource.from_list("eval_datasource_2", samples)
    result = model.evaluate(eval_datasource, value_column="text", record_predictions=True, tags={"test"})
    assert result is not None
    predictions = model.predictions(tag="test")
    assert len(predictions) == 2
    assert all(p.tags == {"test"} for p in predictions)
    assert all(p.expected_label == s["label"] for p, s in zip(predictions, samples))


def test_predict(model: ClassificationModel, label_names: list[str]):
    predictions = model.predict(["Do you love soup?", "Are cats cute?"])
    assert len(predictions) == 2
    assert predictions[0].prediction_id is not None
    assert predictions[1].prediction_id is not None
    assert predictions[0].label == 0
    assert predictions[0].label_name == label_names[0]
    assert 0 <= predictions[0].confidence <= 1
    assert predictions[1].label == 1
    assert predictions[1].label_name == label_names[1]
    assert 0 <= predictions[1].confidence <= 1

    assert predictions[0].logits is not None
    assert predictions[1].logits is not None
    assert len(predictions[0].logits) == 2
    assert len(predictions[1].logits) == 2
    assert predictions[0].logits[0] > predictions[0].logits[1]
    assert predictions[1].logits[0] < predictions[1].logits[1]


def test_predict_disable_telemetry(model: ClassificationModel, label_names: list[str]):
    predictions = model.predict(["Do you love soup?", "Are cats cute?"], save_telemetry=False)
    assert len(predictions) == 2
    assert predictions[0].prediction_id is None
    assert predictions[1].prediction_id is None
    assert predictions[0].label == 0
    assert predictions[0].label_name == label_names[0]
    assert 0 <= predictions[0].confidence <= 1
    assert predictions[1].label == 1
    assert predictions[1].label_name == label_names[1]
    assert 0 <= predictions[1].confidence <= 1


def test_predict_unauthenticated(unauthenticated, model: ClassificationModel):
    with pytest.raises(ValueError, match="Invalid API key"):
        model.predict(["Do you love soup?", "Are cats cute?"])


def test_predict_unauthorized(unauthorized, model: ClassificationModel):
    with pytest.raises(LookupError):
        model.predict(["Do you love soup?", "Are cats cute?"])


def test_predict_constraint_violation(readonly_memoryset: LabeledMemoryset):
    model = ClassificationModel.create(
        "test_model_lookup_count_too_high",
        readonly_memoryset,
        num_classes=2,
        memory_lookup_count=readonly_memoryset.length + 2,
    )
    with pytest.raises(RuntimeError):
        model.predict("test")


def test_record_prediction_feedback(model: ClassificationModel):
    predictions = model.predict(["Do you love soup?", "Are cats cute?"])
    expected_labels = [0, 1]
    model.record_feedback(
        {
            "prediction_id": p.prediction_id,
            "category": "correct",
            "value": p.label == expected_label,
        }
        for expected_label, p in zip(expected_labels, predictions)
    )


def test_record_prediction_feedback_missing_category(model: ClassificationModel):
    prediction = model.predict("Do you love soup?")
    with pytest.raises(ValueError):
        model.record_feedback({"prediction_id": prediction.prediction_id, "value": True})


def test_record_prediction_feedback_invalid_value(model: ClassificationModel):
    prediction = model.predict("Do you love soup?")
    with pytest.raises(ValueError, match=r"Invalid input.*"):
        model.record_feedback({"prediction_id": prediction.prediction_id, "category": "correct", "value": "invalid"})


def test_record_prediction_feedback_invalid_prediction_id(model: ClassificationModel):
    with pytest.raises(ValueError, match=r"Invalid input.*"):
        model.record_feedback({"prediction_id": "invalid", "category": "correct", "value": True})


def test_predict_with_memoryset_override(model: ClassificationModel, hf_dataset: Dataset):
    inverted_labeled_memoryset = LabeledMemoryset.from_hf_dataset(
        "test_memoryset_inverted_labels",
        hf_dataset.map(lambda x: {"label": 1 if x["label"] == 0 else 0}),
        embedding_model=PretrainedEmbeddingModel.GTE_BASE,
    )
    with model.use_memoryset(inverted_labeled_memoryset):
        predictions = model.predict(["Do you love soup?", "Are cats cute?"])
        assert predictions[0].label == 1
        assert predictions[1].label == 0

    predictions = model.predict(["Do you love soup?", "Are cats cute?"])
    assert predictions[0].label == 0
    assert predictions[1].label == 1


def test_predict_with_expected_labels(model: ClassificationModel):
    prediction = model.predict("Do you love soup?", expected_labels=1)
    assert prediction.expected_label == 1


def test_predict_with_expected_labels_invalid_input(model: ClassificationModel):
    # invalid number of expected labels for batch prediction
    with pytest.raises(ValueError, match=r"Invalid input.*"):
        model.predict(["Do you love soup?", "Are cats cute?"], expected_labels=[0])
    # invalid label value
    with pytest.raises(ValueError):
        model.predict("Do you love soup?", expected_labels=5)


def test_last_prediction_with_batch(model: ClassificationModel):
    predictions = model.predict(["Do you love soup?", "Are cats cute?"])
    assert model.last_prediction is not None
    assert model.last_prediction.prediction_id == predictions[-1].prediction_id
    assert model.last_prediction.input_value == "Are cats cute?"
    assert model._last_prediction_was_batch is True


def test_last_prediction_with_single(model: ClassificationModel):
    # Test that last_prediction is updated correctly with single prediction
    prediction = model.predict("Do you love soup?")
    assert model.last_prediction is not None
    assert model.last_prediction.prediction_id == prediction.prediction_id
    assert model.last_prediction.input_value == "Do you love soup?"
    assert model._last_prediction_was_batch is False


@pytest.mark.skipif(
    SKIP_IN_GITHUB_ACTIONS, reason="Skipping explanation test because in CI we don't have Anthropic API key"
)
def test_explain(writable_memoryset: LabeledMemoryset):

    writable_memoryset.analyze(
        {"name": "neighbor", "neighbor_counts": [1, 3]},
        lookup_count=3,
    )

    model = ClassificationModel.create(
        "test_model_for_explain",
        writable_memoryset,
        num_classes=2,
        memory_lookup_count=3,
        description="This is a test model for explain",
    )

    predictions = model.predict(["Do you love soup?", "Are cats cute?"])
    assert len(predictions) == 2

    try:
        explanation = predictions[0].explanation
        print(explanation)
        assert explanation is not None
        assert len(explanation) > 10
        assert "soup" in explanation.lower()
    except Exception as e:
        if "ANTHROPIC_API_KEY" in str(e):
            logging.info("Skipping explanation test because ANTHROPIC_API_KEY is not set on server")
        else:
            raise e
    finally:
        try:
            ClassificationModel.drop("test_model_for_explain")
        except Exception as e:
            logging.info(f"Failed to drop test model for explain: {e}")



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

from enum import Enum
from typing import Any, Literal, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.not_found_error_response_resource_type_0 import NotFoundErrorResponseResourceType0

T = TypeVar("T", bound="NotFoundErrorResponse")


@_attrs_define
class NotFoundErrorResponse:
    """
    Attributes:
        status_code (Literal[404]):
        resource (Union[None, NotFoundErrorResponseResourceType0]):
    """

    status_code: Literal[404]
    resource: Union[None, NotFoundErrorResponseResourceType0]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        status_code = self.status_code

        resource: Union[None, str]
        if isinstance(self.resource, NotFoundErrorResponseResourceType0):
            resource = self.resource.value if isinstance(self.resource, Enum) else self.resource
        else:
            resource = self.resource

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "status_code": status_code,
                "resource": resource,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        status_code = cast(Literal[404], d.pop("status_code"))
        if status_code != 404:
            raise ValueError(f"status_code must match const 404, got '{status_code}'")

        def _parse_resource(data: object) -> Union[None, NotFoundErrorResponseResourceType0]:
            if data is None:
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                resource_type_0 = NotFoundErrorResponseResourceType0(data)

                return resource_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, NotFoundErrorResponseResourceType0], data)

        resource = _parse_resource(d.pop("resource"))

        not_found_error_response = cls(
            status_code=status_code,
            resource=resource,
        )

        not_found_error_response.additional_properties = d
        return not_found_error_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/endpoint_module.py.jinja

The main changes are:
- Update the API call responses to either return the successful response type or raise an error by:
    - Updating the _parse_response function to raise an error if the response status code is not in the 2xx range
- Inject a client into every method via a context manager
"""

from enum import Enum
from http import HTTPStatus
from typing import Any, Dict, List, Union

import httpx

from ...client import _client_context
from ...errors import get_error_for_response
from ...models.memoryset_analysis_response import MemorysetAnalysisResponse
from ...models.task_status import TaskStatus
from ...types import UNSET, Response, Unset


def _get_kwargs(
    name_or_id: str,
    *,
    status: Union[None, TaskStatus, Unset] = UNSET,
    limit: Union[None, Unset, int] = 100,
    offset: Union[None, Unset, int] = 0,
) -> dict[str, Any]:
    params: Dict[str, Any] = {}

    json_status: Union[None, Unset, str]
    if isinstance(status, Unset):
        json_status = UNSET
    elif isinstance(status, TaskStatus):
        json_status = status.value if isinstance(status, Enum) else status
    else:
        json_status = status
    params["status"] = json_status

    json_limit: Union[None, Unset, int]
    if isinstance(limit, Unset):
        json_limit = UNSET
    else:
        json_limit = limit
    params["limit"] = json_limit

    json_offset: Union[None, Unset, int]
    if isinstance(offset, Unset):
        json_offset = UNSET
    else:
        json_offset = offset
    params["offset"] = json_offset

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": f"/memoryset/{name_or_id}/analysis",
        "params": params,
    }

    return _kwargs


def _parse_response(*, response: httpx.Response) -> List["MemorysetAnalysisResponse"]:
    if response.status_code == 200:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = MemorysetAnalysisResponse.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200
    if response.status_code == 401:
        raise get_error_for_response(response)
    if response.status_code == 500:
        raise get_error_for_response(response)
    if response.status_code == 503:
        raise get_error_for_response(response)
    if response.status_code == 404:
        raise get_error_for_response(response)
    else:
        raise get_error_for_response(response)


def _build_response(*, response: httpx.Response) -> Response[List["MemorysetAnalysisResponse"]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    name_or_id: str,
    *,
    status: Union[None, TaskStatus, Unset] = UNSET,
    limit: Union[None, Unset, int] = 100,
    offset: Union[None, Unset, int] = 0,
) -> Response[List["MemorysetAnalysisResponse"]]:
    """List Analyses

    Args:
        name_or_id (str):
        status (Union[None, TaskStatus, Unset]):
        limit (Union[None, Unset, int]):  Default: 100.
        offset (Union[None, Unset, int]):  Default: 0.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[List['MemorysetAnalysisResponse']]
    """

    kwargs = _get_kwargs(
        name_or_id=name_or_id,
        status=status,
        limit=limit,
        offset=offset,
    )

    with _client_context() as client:
        response = client.get_httpx_client().request(
            **kwargs,
        )

        return _build_response(response=response)


def sync(
    name_or_id: str,
    *,
    status: Union[None, TaskStatus, Unset] = UNSET,
    limit: Union[None, Unset, int] = 100,
    offset: Union[None, Unset, int] = 0,
) -> List["MemorysetAnalysisResponse"]:
    """List Analyses

    Args:
        name_or_id (str):
        status (Union[None, TaskStatus, Unset]):
        limit (Union[None, Unset, int]):  Default: 100.
        offset (Union[None, Unset, int]):  Default: 0.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        List['MemorysetAnalysisResponse']
    """

    return sync_detailed(
        name_or_id=name_or_id,
        status=status,
        limit=limit,
        offset=offset,
    ).parsed


async def asyncio_detailed(
    name_or_id: str,
    *,
    status: Union[None, TaskStatus, Unset] = UNSET,
    limit: Union[None, Unset, int] = 100,
    offset: Union[None, Unset, int] = 0,
) -> Response[List["MemorysetAnalysisResponse"]]:
    """List Analyses

    Args:
        name_or_id (str):
        status (Union[None, TaskStatus, Unset]):
        limit (Union[None, Unset, int]):  Default: 100.
        offset (Union[None, Unset, int]):  Default: 0.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[List['MemorysetAnalysisResponse']]
    """

    kwargs = _get_kwargs(
        name_or_id=name_or_id,
        status=status,
        limit=limit,
        offset=offset,
    )

    with _client_context() as client:
        response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    name_or_id: str,
    *,
    status: Union[None, TaskStatus, Unset] = UNSET,
    limit: Union[None, Unset, int] = 100,
    offset: Union[None, Unset, int] = 0,
) -> List["MemorysetAnalysisResponse"]:
    """List Analyses

    Args:
        name_or_id (str):
        status (Union[None, TaskStatus, Unset]):
        limit (Union[None, Unset, int]):  Default: 100.
        offset (Union[None, Unset, int]):  Default: 0.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        List['MemorysetAnalysisResponse']
    """

    return (
        await asyncio_detailed(
            name_or_id=name_or_id,
            status=status,
            limit=limit,
            offset=offset,
        )
    ).parsed



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

from typing import Any, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="LabeledMemorysetUpdate")


@_attrs_define
class LabeledMemorysetUpdate:
    """
    Attributes:
        label_names (Union[List[str], None, Unset]):
        description (Union[None, Unset, str]):
    """

    label_names: Union[List[str], None, Unset] = UNSET
    description: Union[None, Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        label_names: Union[List[str], None, Unset]
        if isinstance(self.label_names, Unset):
            label_names = UNSET
        elif isinstance(self.label_names, list):
            label_names = self.label_names

        else:
            label_names = self.label_names

        description: Union[None, Unset, str]
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if label_names is not UNSET:
            field_dict["label_names"] = label_names
        if description is not UNSET:
            field_dict["description"] = description

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()

        def _parse_label_names(data: object) -> Union[List[str], None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                label_names_type_0 = cast(List[str], data)

                return label_names_type_0
            except:  # noqa: E722
                pass
            return cast(Union[List[str], None, Unset], data)

        label_names = _parse_label_names(d.pop("label_names", UNSET))

        def _parse_description(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        description = _parse_description(d.pop("description", UNSET))

        labeled_memoryset_update = cls(
            label_names=label_names,
            description=description,
        )

        labeled_memoryset_update.additional_properties = d
        return labeled_memoryset_update

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

import datetime
from typing import Any, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="CountPredictionsRequest")


@_attrs_define
class CountPredictionsRequest:
    """
    Attributes:
        model_id (Union[None, Unset, str]):
        tag (Union[None, Unset, str]):
        prediction_ids (Union[List[str], None, Unset]):
        start_timestamp (Union[None, Unset, datetime.datetime]):
        end_timestamp (Union[None, Unset, datetime.datetime]):
    """

    model_id: Union[None, Unset, str] = UNSET
    tag: Union[None, Unset, str] = UNSET
    prediction_ids: Union[List[str], None, Unset] = UNSET
    start_timestamp: Union[None, Unset, datetime.datetime] = UNSET
    end_timestamp: Union[None, Unset, datetime.datetime] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        model_id: Union[None, Unset, str]
        if isinstance(self.model_id, Unset):
            model_id = UNSET
        else:
            model_id = self.model_id

        tag: Union[None, Unset, str]
        if isinstance(self.tag, Unset):
            tag = UNSET
        else:
            tag = self.tag

        prediction_ids: Union[List[str], None, Unset]
        if isinstance(self.prediction_ids, Unset):
            prediction_ids = UNSET
        elif isinstance(self.prediction_ids, list):
            prediction_ids = self.prediction_ids

        else:
            prediction_ids = self.prediction_ids

        start_timestamp: Union[None, Unset, str]
        if isinstance(self.start_timestamp, Unset):
            start_timestamp = UNSET
        elif isinstance(self.start_timestamp, datetime.datetime):
            start_timestamp = self.start_timestamp.isoformat()
        else:
            start_timestamp = self.start_timestamp

        end_timestamp: Union[None, Unset, str]
        if isinstance(self.end_timestamp, Unset):
            end_timestamp = UNSET
        elif isinstance(self.end_timestamp, datetime.datetime):
            end_timestamp = self.end_timestamp.isoformat()
        else:
            end_timestamp = self.end_timestamp

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if model_id is not UNSET:
            field_dict["model_id"] = model_id
        if tag is not UNSET:
            field_dict["tag"] = tag
        if prediction_ids is not UNSET:
            field_dict["prediction_ids"] = prediction_ids
        if start_timestamp is not UNSET:
            field_dict["start_timestamp"] = start_timestamp
        if end_timestamp is not UNSET:
            field_dict["end_timestamp"] = end_timestamp

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()

        def _parse_model_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        model_id = _parse_model_id(d.pop("model_id", UNSET))

        def _parse_tag(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        tag = _parse_tag(d.pop("tag", UNSET))

        def _parse_prediction_ids(data: object) -> Union[List[str], None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                prediction_ids_type_0 = cast(List[str], data)

                return prediction_ids_type_0
            except:  # noqa: E722
                pass
            return cast(Union[List[str], None, Unset], data)

        prediction_ids = _parse_prediction_ids(d.pop("prediction_ids", UNSET))

        def _parse_start_timestamp(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                start_timestamp_type_0 = isoparse(data)

                return start_timestamp_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        start_timestamp = _parse_start_timestamp(d.pop("start_timestamp", UNSET))

        def _parse_end_timestamp(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                end_timestamp_type_0 = isoparse(data)

                return end_timestamp_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        end_timestamp = _parse_end_timestamp(d.pop("end_timestamp", UNSET))

        count_predictions_request = cls(
            model_id=model_id,
            tag=tag,
            prediction_ids=prediction_ids,
            start_timestamp=start_timestamp,
            end_timestamp=end_timestamp,
        )

        count_predictions_request.additional_properties = d
        return count_predictions_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
